/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.filestore.factory;

import org.globus.wsrf.security.SecurityManager;

import org.globus.wsrf.container.Lock;
import org.globus.wsrf.container.LockManager;

import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.InvalidResourceKeyException;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.utils.AddressingUtils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.tutorial.types.filestore.Request;
import org.globus.tutorial.types.filestore.FileStoreException;

import org.apache.axis.components.uuid.UUIDGen;
import org.apache.axis.components.uuid.UUIDGenFactory;

import java.util.HashMap;

import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.tutorial.impl.filestore.Constants;

import org.globus.wsrf.utils.FilePersistenceHelper;

import org.globus.util.Util;

import java.io.File;
import java.io.PrintWriter;
import java.io.FileWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

public class FileStoreFactoryHome extends ResourceHomeImpl {

    static Log logger = 
        LogFactory.getLog(FileStoreFactoryHome.class.getName());

    protected FilePersistenceHelper persistenceHelper;

    protected static String ID = "FileServiceFactoryHome";

    public static UUIDGen uuidGen = UUIDGenFactory.getUUIDGen();
    protected String configFile;
    protected String mappingFile = "keyDNMap";

    // could use DN as key, but the FileService is insecure, so
    // getting EPR would be too easy. So maintaining list of DNs and
    // key map.
    HashMap dnKeyMap = new HashMap();


    // For calling from FileService code.
    public Resource find(String resourceId) throws ResourceException {
        return super.find(getKey(resourceId));
    }

    public Resource find(ResourceKey key)
        throws ResourceException {

        String callerDN = SecurityManager.getManager().getCaller();
        return findResource(callerDN);
    }

    public Resource findResource(String callerDN) throws ResourceException {

        if (callerDN == null) {
            throw new InvalidResourceKeyException("Caller DN is null "
                                                  + "cannot constuct key");
        }

        ResourceKey keyForCaller = (ResourceKey)dnKeyMap.get(callerDN);
        if (keyForCaller == null) {
            throw new InvalidResourceKeyException("Caller does not have "
                                                  + "valid key");
        }

         return super.find(keyForCaller);
    }

    public EndpointReferenceType create(Request request, String callerDN) 
        throws NoSuchResourceException, ResourceException, FileStoreException {
        
        logger.debug("Create for " + callerDN);
        FileStoreFactoryResource resource = null;

        // try to get key
        ResourceKey key = (ResourceKey)dnKeyMap.get(callerDN);
        if (key != null) {
            
            logger.debug("Key present " + key);
            resource = (FileStoreFactoryResource)find(key);
            // should not be if destroy is handed well
            if (resource == null) {
                throw new NoSuchResourceException();
            }
        } else {
            // FIXME: this code is  copied to
            // PersistentFileStoreFactoryHome. Needs to be refactored better to
            // resue code.
            logger.debug("creating new factory resource");
            resource = new FileStoreFactoryResource(callerDN, 
                                                    uuidGen.nextUUID(),
                                                    this.configFile);
            // Type casting to string, should not change jndi file
            String id = (String)resource.getID();
            key = getKey(id);
            add(key, resource);
            dnKeyMap.put(callerDN, key);

            // This is just for house keeping. In case clients over
            // run quota and don't have EPR saved to delete
            // files. This mechanism lets us identify which directory
            // to clean up given a DN.
            FileWriter writer = null;
            PrintWriter printWriter = null;
            try {
                writer = new FileWriter(mappingFile);
                printWriter = new PrintWriter(writer);
                printWriter.write(callerDN + " = " + id);
            } catch (IOException exp) {
                System.err.println("Error writing mapping out to file" +
                                   exp.getMessage());
            } finally {
                try {
                    if (writer != null) {
                    writer.close();
                    }
                    if (printWriter != null) {
                        printWriter.close();
                    }
                } catch (Exception exp) {}
            }
        }
        
        ResourceKey fileServiceKey = resource.create(request);
        // Create EPR
        EndpointReferenceType epr = null;
        try {
            String addr = ServiceHost.getBaseURL() + getPath();
            epr = AddressingUtils.createEndpointReference(addr, 
                                                          fileServiceKey);
        } catch (Exception exp) {
            throw new ResourceException(exp);
        }
        logger.debug("EPR for resource: " + epr);
        return epr;
    }

    public String getPath() {
        return Constants.SERVICE_PATH;
    }

    public FileStoreFactoryResource getResource(String resourceId) 
        throws ResourceException {
        SimpleResourceKey key = 
            new SimpleResourceKey(keyTypeName, resourceId);
        return (FileStoreFactoryResource)find(key);
    }
    
    public void setConfigFile(String configFile) {
        this.configFile = configFile;
    }
    
    public String getConfigFile() {
        return this.configFile;
    }

    protected ResourceKey getKey(String resourceId) {
        return new SimpleResourceKey(this.keyTypeName, resourceId);
    }
}
