/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.filestore.factory;

import org.globus.tutorial.impl.filestore.service.FileServiceHome;

import org.globus.tutorial.impl.filestore.Constants;

import org.globus.tutorial.types.filestore.Request;
import org.globus.tutorial.types.filestore.RequestFileType;
import org.globus.tutorial.types.filestore.FileStoreException;


import javax.xml.namespace.QName;

import org.globus.security.gridmap.GridMap;

import javax.naming.InitialContext;

import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceIdentifier;
import org.globus.wsrf.ResourceKey;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.axis.components.uuid.UUIDGen;
import org.apache.axis.components.uuid.UUIDGenFactory;

import org.globus.wsrf.security.SecureResource;
import org.globus.wsrf.impl.security.descriptor.ResourceSecurityDescriptor;
import org.globus.wsrf.impl.security.descriptor.SecurityDescriptorException;

import java.util.Properties;
import java.util.Vector;
import java.util.List;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

import org.globus.wsrf.impl.security.descriptor.GSITransportAuthMethod;
import org.globus.wsrf.impl.security.descriptor.GSISecureConvAuthMethod;
import org.globus.wsrf.impl.security.descriptor.GSISecureMsgAuthMethod;

import org.globus.wsrf.InvalidResourceKeyException;
import org.globus.wsrf.RemoveCallback;
import org.globus.wsrf.PersistenceCallback;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;

import org.globus.wsrf.impl.SimpleResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertySet;

import org.globus.wsrf.utils.FilePersistenceHelper;

import org.globus.util.Util;

public class FileStoreFactoryResource 
    implements Resource, ResourceIdentifier, SecureResource, 
               ResourceProperties {

    static Log logger = 
        LogFactory.getLog(FileStoreFactoryResource.class.getName());

    static final String QUOTA_PROPERTY = "quota";
    static final String QUOTA_VALUE = "10000";

    public static final String QUOTA_RP = "spaceQuota";
    public static final String SPACE_LEFT_RP = "spaceLeft";

    protected String callerDN;
    protected ResourceSecurityDescriptor desc = null;
    protected String resourceId;
    protected File configFile = null;
    protected Long configFileTS = null;
    protected long quota = 0;
    protected long unusedQuota = 0;


    ResourcePropertySet propSet;
    QName RP_SET = new QName(Constants.NS, "FileFactoryRPSet");

    public FileStoreFactoryResource() {
    }

    public FileStoreFactoryResource(String callerDN, String id, 
                                    String configFilename) 
        throws FileStoreException, ResourceException {

        if ((callerDN == null) || (id == null) || (configFilename == null)) {
            throw new IllegalArgumentException("CallerDN, Resouece id and"
                                               + " config name cannot be " 
                                               + " null");
        }
        
        this.callerDN = callerDN;
        this.resourceId = id;
        this.configFile = new File(configFilename);

        this.desc = new ResourceSecurityDescriptor();
        this.desc.setAuthz("gridmap");
        GridMap gridMap = new GridMap();
        // dummy mapping since gridmap authorization is not used.
        gridMap.map(this.callerDN, "temp");
        this.desc.setGridMap(gridMap);

        List methods = new Vector();
        methods.add(GSITransportAuthMethod.BOTH);
        methods.add(GSISecureConvAuthMethod.BOTH);
        methods.add(GSISecureMsgAuthMethod.BOTH);

        try {
            this.desc.setAuthMethods(methods);
        } catch (SecurityDescriptorException exp) {
            throw Constants.makeFault(exp.getMessage(), exp);
        }

        updateQuota();
    }


    public ResourceKey create(Request request) 
        throws ResourceException, FileStoreException {

        long sizeOfFile = request.getFileSizeInBytes();
        updateQuota();
        if (sizeOfFile > this.unusedQuota) {
            throw Constants.makeFault("Exceeding quota " + this.quota
                                         + " unused " + this.unusedQuota);
        }
        this.unusedQuota = this.unusedQuota - sizeOfFile;
        setResourceProperties();

        FileServiceHome home = getServiceHome();
        return home.create(request, this.resourceId);
    }
    
    // ResourceIdentifier interface
    public Object getID() {
        return this.resourceId;
    }
    
    // Secure resource interface
    public ResourceSecurityDescriptor getSecurityDescriptor() {
        return this.desc;
    }

    protected String getPath() {
        return Constants.SERVICE_PATH;
    }

    protected FileServiceHome getServiceHome() throws FileStoreException {
        try {
            InitialContext context = new InitialContext();
            return (FileServiceHome)context
                .lookup(org.globus.wsrf.Constants.JNDI_SERVICES_BASE_NAME +
                        getPath() +
                        org.globus.wsrf.Constants.HOME_NAME);
        } catch (Exception exp) {
            throw Constants.makeFault(exp.getMessage(), exp);
        }
    }

    protected void updateQuota() throws FileStoreException {
        Long ts = new Long(this.configFile.lastModified());
        if (configFileTS != null) {
            // has not been modified
            if (ts.compareTo(configFileTS) == 0) {
                return;
            }
        }
        Properties prop = new Properties();
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(this.configFile);
            prop.load(fis);
        } catch (FileNotFoundException fnfe) {
            Constants.makeFault(fnfe.getMessage(), fnfe);
        } catch (IOException exp) {
            Constants.makeFault(exp.getMessage(), exp);
        } finally {
            if (fis != null) {
                    try {
                        fis.close();
                    } catch (Exception e) {}
            }
        }
     
        // this will not be null since default value is used if not present.
        String quotaProp = prop.getProperty(QUOTA_PROPERTY, QUOTA_VALUE);
        long newQuota = new Long(quotaProp).longValue();
        this.unusedQuota = this.unusedQuota + (newQuota - this.quota);
        this.quota = newQuota;
        this.configFileTS = ts;
        logger.info("Unused quota " + this.unusedQuota + " quota "
                           + this.quota);
        setResourceProperties();
    }

    // Resource properties
    public ResourcePropertySet getResourcePropertySet() {
        return this.propSet;
    }

    protected void setResourceProperties() {

        this.propSet = new SimpleResourcePropertySet(RP_SET);
        QName qName = new QName(Constants.NS, QUOTA_RP);
        ResourceProperty prop = new SimpleResourceProperty(qName);
        prop.add(new Long(this.quota));
        this.propSet.add(prop);
        qName = new QName(Constants.NS, SPACE_LEFT_RP);
        prop = new SimpleResourceProperty(qName);
        prop.add(new Long(unusedQuota));
        this.propSet.add(prop);
    }

    public boolean checkQuota(long diff) throws FileStoreException {
        
        updateQuota();
        long newUnusedQuota = this.unusedQuota - diff;
        if (newUnusedQuota > 0) {
            this.unusedQuota = newUnusedQuota;
            setResourceProperties();
            return true;
        }
        return false;
    }

    // Called by FileService Resource.
    public void addToQuota(long bytes) {
        this.unusedQuota = this.unusedQuota + bytes;
        if (this.unusedQuota > this.quota) {
	    logger.warn("quota reached by " + this.callerDN);
            this.unusedQuota = this.quota;
        }
        setResourceProperties();
    }
}
