/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.filestore.factory;

import org.globus.tutorial.impl.filestore.service.FileServiceHome;

import org.globus.tutorial.impl.filestore.Constants;

import org.globus.tutorial.types.filestore.Request;
import org.globus.tutorial.types.filestore.RequestFileType;
import org.globus.tutorial.types.filestore.FileStoreException;


import javax.xml.namespace.QName;

import org.globus.security.gridmap.GridMap;

import javax.naming.InitialContext;

import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceIdentifier;
import org.globus.wsrf.ResourceKey;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.axis.components.uuid.UUIDGen;
import org.apache.axis.components.uuid.UUIDGenFactory;

import org.globus.wsrf.security.SecureResource;
import org.globus.wsrf.impl.security.descriptor.ResourceSecurityDescriptor;
import org.globus.wsrf.impl.security.descriptor.SecurityDescriptorException;

import java.util.Properties;
import java.util.Vector;
import java.util.List;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

import org.globus.wsrf.impl.security.descriptor.GSITransportAuthMethod;
import org.globus.wsrf.impl.security.descriptor.GSISecureConvAuthMethod;
import org.globus.wsrf.impl.security.descriptor.GSISecureMsgAuthMethod;

import org.globus.wsrf.InvalidResourceKeyException;
import org.globus.wsrf.RemoveCallback;
import org.globus.wsrf.PersistenceCallback;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;

import org.globus.wsrf.impl.SimpleResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertySet;

import org.globus.wsrf.utils.FilePersistenceHelper;

import org.globus.util.Util;

public class PersistentFileStoreFactoryResource 
    extends FileStoreFactoryResource implements PersistenceCallback, 
                                                RemoveCallback {

    static Log logger = 
        LogFactory.getLog(PersistentFileStoreFactoryResource.class.getName());

    private FilePersistenceHelper persistenceHelper;

    public PersistentFileStoreFactoryResource() {
    }

    public PersistentFileStoreFactoryResource(String callerDN, String id, 
                                              String configFilename) 
        throws FileStoreException, ResourceException {

        super(callerDN, id, configFilename);
        
        store();
    }

    // PersistenceCallback interface
    public synchronized void store() throws ResourceException {
        FileOutputStream fos = null;
        File tmpFile = null;
        try {
            tmpFile = File.createTempFile("fileStoreFactory", ".tmp",
                                          getPersistenceHelper()
                                          .getStorageDirectory());
            
            if (!Util.setFilePermissions(tmpFile.getAbsolutePath(), 700)) {
                logger.warn("Check file permissions " 
                            + tmpFile.getAbsolutePath());
            }
            fos = new FileOutputStream(tmpFile);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            oos.writeObject(this.callerDN);
            oos.writeObject(this.desc);
            oos.writeObject(this.configFile);
            oos.writeObject(this.configFileTS);
            oos.writeObject(new Long(this.quota));
            oos.writeObject(new Long(this.unusedQuota));
        } catch (Exception e) {
            if (tmpFile != null) {
                tmpFile.delete();
            }
            throw new ResourceException("Error storing resource", e);
        } finally {
            if (fos != null) {
                try { fos.close();} catch (Exception ee) {}
            }
        }

        // Overwrite old file
        File file = getKeyAsFile(this.resourceId);
        if (file.exists()) {
            file.delete();
        }
        
        if (!tmpFile.renameTo(file)) {
            tmpFile.delete();
            throw new ResourceException("Error storing resource");
        }

        // Check if rename to changes the file permissions.
        if (!Util.setFilePermissions(file.getAbsolutePath(), 700)) {
            logger.warn("Check file permissions " + file.getAbsolutePath());
        }
    }

    public void load(ResourceKey key) throws ResourceException,
                                             NoSuchResourceException,
                                             InvalidResourceKeyException {

        this.resourceId = (String)key.getValue();
        File file = getKeyAsFile(this.resourceId);
        if (!file.exists()) {
            throw new NoSuchResourceException();
        }

        FileInputStream fis = null;
        try {
            fis = new FileInputStream(file);
            ObjectInputStream ois = new ObjectInputStream(fis);
            this.callerDN = (String)ois.readObject();
            this.desc = (ResourceSecurityDescriptor)ois.readObject();
            this.configFile = (File)ois.readObject();
            this.configFileTS = (Long)ois.readObject();
            this.quota = ((Long)ois.readObject()).longValue();
            this.unusedQuota = ((Long)ois.readObject()).longValue();
        } catch (Exception e) {
            throw new ResourceException("Error loading resource", e);
        } finally {
            if (fis != null) {
                try { fis.close(); } catch (Exception ee) {}
            }
        }
        setResourceProperties();
    }

    private File getKeyAsFile(Object key)
        throws InvalidResourceKeyException {
        if (key instanceof String) {
            return getPersistenceHelper().getKeyAsFile(key);
        } else {
            throw new InvalidResourceKeyException();
        }
    }

    protected synchronized FilePersistenceHelper getPersistenceHelper() {
        if (this.persistenceHelper == null) {
            try {
                this.persistenceHelper =
                    new FilePersistenceHelper(getClass(), ".ser");
            } catch (Exception e) {
                throw new RuntimeException(e.getMessage());
            }
        }
        return this.persistenceHelper;
    }

    protected String getPath() {
        return Constants.PERSIST_SERVICE_PATH;
    }

    public ResourceKey create(Request request) 
        throws ResourceException, FileStoreException {

        ResourceKey key = super.create(request);

        store();

        return key;
    }

    // Called by FileService Resource.
    public void addToQuota(long bytes) {
        super.addToQuota(bytes);
        try {
            store();
        } catch (ResourceException exp) {
            throw new RuntimeException("Error storing resource" 
                                       + exp.getMessage());
        }
    }

    // RemoveCallback
    public void remove() throws ResourceException {
        getPersistenceHelper().remove(this.resourceId);
        getFactoryHome().remove();
    }

    protected PersistentFileStoreFactoryHome getFactoryHome() 
        throws ResourceException {
        try {
            InitialContext context = new InitialContext();
            return (PersistentFileStoreFactoryHome)context
                .lookup(org.globus.wsrf.Constants.JNDI_SERVICES_BASE_NAME +
                        Constants.PERSIST_FACTORY_PATH +
                        org.globus.wsrf.Constants.HOME_NAME);
        } catch (Exception exp) {
            throw new ResourceException(exp);
        }
    }
}

