/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.filestore.service;

import javax.xml.namespace.QName;

import school.tutorial.writer.XMLWriteFile;
import school.tutorial.writer.XMLWriteFileException;
import school.tutorial.parser.XMLParser;
import school.tutorial.parser.XMLParser1D;
import school.tutorial.parser.XMLParser2D;
import school.tutorial.parser.XMLParser3D;

import org.globus.tutorial.impl.filestore.Constants;

import org.globus.tutorial.impl.filestore.factory.FileStoreFactoryResource;
import org.globus.tutorial.impl.filestore.factory.FileStoreFactoryHome;

import org.globus.security.gridmap.GridMap;

import javax.naming.InitialContext;

import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.utils.AddressingUtils;
import org.globus.wsrf.RemoveCallback;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceIdentifier;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceLifetime;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.wsrf.PersistenceCallback;
import org.globus.wsrf.impl.SimpleResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.wsrf.InvalidResourceKeyException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.axis.components.uuid.UUIDGen;
import org.apache.axis.components.uuid.UUIDGenFactory;

import org.globus.tutorial.types.filestore.Point;
import org.globus.tutorial.types.filestore.Triple;
import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.Request;
import org.globus.tutorial.types.filestore.FileStoreException;
import org.globus.tutorial.types.filestore.RequestFileType;
import org.globus.tutorial.types.filestore.ArrayOfDouble;

import java.util.Calendar;
import java.util.Vector;
import java.util.List;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;

import org.globus.wsrf.utils.XmlUtils;

import org.w3c.dom.Document;

import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;

import org.globus.wsrf.utils.FilePersistenceHelper;

import org.globus.util.Util;

public class PersistentFileServiceResource extends FileServiceResource
    implements RemoveCallback, PersistenceCallback {

    static Log logger = 
        LogFactory.getLog(PersistentFileServiceResource.class.getName());

    private FilePersistenceHelper persistenceHelper;

    public PersistentFileServiceResource() {
    }

    public PersistentFileServiceResource(Request request, String id,
                                         String factoryId,
                                         String fileStoreRoot_) 
        throws ResourceException {

        super(request, id, factoryId, fileStoreRoot_);
        store();
    }

    public synchronized void store() throws ResourceException {
        FileOutputStream fos = null;
        File tmpFile = null;
        try {
            tmpFile = File.createTempFile("fileStore", ".tmp",
                                          getPersistenceHelper()
                                          .getStorageDirectory());
            
            if (!Util.setFilePermissions(tmpFile.getAbsolutePath(), 700)) {
                logger.warn("Check file permissions " 
                            + tmpFile.getAbsolutePath());
            }
            fos = new FileOutputStream(tmpFile);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            oos.writeObject(this.creationTime);
            oos.writeObject(this.terminationTime);
            oos.writeObject(this.fileType);
            oos.writeObject(this.bytes);
            oos.writeObject(this.factoryId);
            oos.writeObject(this.fileStoreRoot);
            oos.writeObject(this.size);
        } catch (Exception e) {
            if (tmpFile != null) {
                tmpFile.delete();
            }
            throw new ResourceException("Error storing resource", e);
        } finally {
            if (fos != null) {
                try { fos.close();} catch (Exception ee) {}
            }
        }

        // Overwrite old file
        File file = getKeyAsFile(this.resourceId);
        if (file.exists()) {
            file.delete();
        }
        
        if (!tmpFile.renameTo(file)) {
            tmpFile.delete();
            throw new ResourceException("Error storing resource");
        }

        // Check if rename to changes the file permissions.
        if (!Util.setFilePermissions(file.getAbsolutePath(), 700)) {
            logger.warn("Check file permissions " + file.getAbsolutePath());
        }
    }

    public void load(ResourceKey key) throws ResourceException,
                                             NoSuchResourceException,
                                             InvalidResourceKeyException {

        this.resourceId = (String)key.getValue();
        File file = getKeyAsFile(this.resourceId);
        if (!file.exists()) {
            throw new NoSuchResourceException();
        }

        FileInputStream fis = null;
        try {
            fis = new FileInputStream(file);
            ObjectInputStream ois = new ObjectInputStream(fis);
            this.creationTime = (Calendar)ois.readObject();
            this.terminationTime = (Calendar)ois.readObject();
            this.fileType = (RequestFileType)ois.readObject();
            this.bytes = (Long)ois.readObject();
            this.factoryId = (String)ois.readObject();
            this.fileStoreRoot = (String)ois.readObject();
            this.size = (Integer)ois.readObject();
        } catch (Exception e) {
            throw new ResourceException("Error loading resource", e);
        } finally {
            if (fis != null) {
                try { fis.close(); } catch (Exception ee) {}
            }
        }
        setUpDirectory();
        setResourceProperties();
    }

    protected String getPath() {
        return Constants.PERSIST_FACTORY_PATH;
    }

    private File getKeyAsFile(Object key)
        throws InvalidResourceKeyException {
        if (key instanceof String) {
            return getPersistenceHelper().getKeyAsFile(key);
        } else {
            throw new InvalidResourceKeyException();
        }
    }

    protected synchronized FilePersistenceHelper getPersistenceHelper() {
        if (this.persistenceHelper == null) {
            try {
                this.persistenceHelper =
                    new FilePersistenceHelper(getClass(), ".ser");
            } catch (Exception e) {
                throw new RuntimeException(e.getMessage());
            }
        }
        return this.persistenceHelper;
    }
    
    // Resource lifetime interface
    public void setTerminationTime(Calendar _terminationTime) {
        this.terminationTime   = _terminationTime;
        try {
            store();
        } catch (ResourceException exp) {
            throw new RuntimeException(exp.getMessage());
        }
    }


    // RemoveCallback
    public void remove() throws ResourceException {
        getPersistenceHelper().remove(this.resourceId);
	delete();
    }

    public void store(Data parameters) throws FileStoreException {
        
        super.store(parameters);
        try {
            store();
        } catch (ResourceException exp) {
            throw Constants.makeFault("Error storing resource", exp);
        }
    }

}

