/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.randgen.service;

import java.rmi.RemoteException;

import org.globus.tutorial.types.randgen.RandomGenResponse;
import org.globus.tutorial.types.randgen.RandomGenReq;
import org.globus.tutorial.types.randgen.RandGeneratorException;

import org.apache.axis.message.addressing.EndpointReferenceType;

import javax.xml.rpc.Stub;

import java.util.Random;

import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.ArrayOfDouble;

import org.globus.tutorial.filestore.FileService;
import org.globus.tutorial.filestore.service.FileServiceAddressingLocator;
import org.globus.tutorial.types.filestore.FileStoreException;

import javax.xml.rpc.ServiceException;

import org.globus.wsrf.utils.FaultHelper;

/**
 * A class which generates a set of random
 * points (x,y) within a specified 2D square.
 */
public class RandGen {

    /** Generates a set of random number pairs within a specified range defined
     * by a square (parallel to the x and y axes).
     * @param req the request object containing the inputs
     * @return the response object
     */
    public RandomGenResponse generate(RandomGenReq req)  
	throws RandGeneratorException, ServiceException {

        Box bound = null;
        try {
            bound = new Box(req.getBox_1());
        } catch (BoxException exp) {
            throw new RandGeneratorException(exp.getMessage());
        }

        int count = req.getInt_2();
        EndpointReferenceType f = req.getURLFileName2D_3();

	if (count <= 0) {
            throw new RandGeneratorException("RandGen: count must"+
                                             " be >0 not "+count);
        }

        String error = "RandGen: ";
	Random rand = new Random();
        double ucx = bound.getUpperCornerX();
        double lcx = bound.getLowerCornerX();
        double ucy = bound.getUpperCornerY();
        double lcy = bound.getLowerCornerY();
	double[] x = new double[count];
	double[] y = new double[count];

        for (int i = 0; i < count; i++) {
            x[i] = lcx+rand.nextDouble()*(ucx-lcx);
	    y[i] = lcy+rand.nextDouble()*(ucy-lcy);
        }
	ArrayOfDouble[] outArray = {new ArrayOfDouble(x),
				    new ArrayOfDouble(y)};
        Data request2D = new Data(outArray);
	try{
            FileServiceAddressingLocator loc = 
                new FileServiceAddressingLocator();
            FileService service = loc.getFileServicePort(f);
	    service.store(request2D);
            return new RandomGenResponse();
	} catch(FileStoreException fse) {
            FaultHelper fh = new FaultHelper(fse);
            String er = fh.getDescriptionAsString();
            if (er.equals("This operation can only be used with 2D files")) {
                error += "Input file should be a 2D file";
            } else {
                error += "FileStore: "+er;
            }
	    error += fse.getMessage();
	} catch(RemoteException re){
	    error += "Remote error accessing FileStore "+re.getMessage();
	}

	throw new RandGeneratorException(error);
    }
}
