/*
 * Pillar.java 
 *
 * Created on 14th July 2005, 10:16
 */
package org.globus.tutorial.impl.surface.service;

import java.io.BufferedReader;
import java.io.IOException;

/** Defines a pillar of wisdom. The pillar has a plaque on top of it
 * on which there is some text which is either raised or lowered in
 * relation to the plaque. The plaque is raised above the top of the
 * pillar.
 * @author Sara
 * Modified by GT4 team.
 */

public class Pillar{

    private static final int pixel_steps_y = 8; 

    // values describing pillar and plaque read in from file
    double height, x_width, y_width, blh_x, blh_y, 
	pixel_step, offset_x, offset_y,
	x_dim, y_dim, z_dim, pixel_height;

    double baseHeight;  //calculated once after data read in

    private int pixel_steps_x; 

    private boolean[][] pixels; //vertical dimension first & true => pixel

    /** Reads in the text to go on top of the pillar. The text is
     * encrypted in a file.
     * @param br the input buffer
     * @param se the StringEncrypter used to decrypt the text.
     */
    public void setPixels( BufferedReader br, StringEncrypter se ) 
        throws IOException, EncryptionException {

	String line = se.decrypt(br.readLine());
	pixel_steps_x = Integer.parseInt(line);
	pixels = new boolean[pixel_steps_y][pixel_steps_x];
	for (int pixRow = pixel_steps_y -1; pixRow >= 0; pixRow--) { 
	    String rowOfPixels = se.decrypt(br.readLine());
	    for (int pix = 0; pix < pixel_steps_x; pix++) { 
		if(rowOfPixels.charAt(pix+1) == '#')
		    pixels[pixRow][pix] = true;
		else if(rowOfPixels.charAt(pix+1) == ' ')
		    pixels[pixRow][pix] = false;
		else
		    throw new IOException("Something wrong with "+
					  "pixels text file" );
	    }
	}
    }
    
    /** Checks if a point is located inside the pillar
     * @param x the x coordinate of the point
     * @param y the y coordinate of the point
     * @return true if the point is inside the pillar
     */
    public boolean inside( double x, double y ) {
	return x >=  blh_x && x <= blh_x+x_width && 
	    y >= blh_y && y <= blh_y+y_width;
    }

    /** Calculates the height of the pillar at a particular point
     * @param x the x coordinate of the point
     * @param y the y coordinate of the point
     * @return the height of the pillar
     */
    public double pillarSurface( double x, double y) {  
	// already know inside is true
	double px = x - blh_x;     //use coordinates relative to pillar
	double py = y - blh_y;
	if (onPlaque( px, py )) {
	    return baseHeight*(1. + height + z_dim + 
		pixelSurface( px-offset_x, py-offset_y));
	}
	return Math.abs(baseHeight)*(1. + height);
    }

    /** Checks whether a point is located on the plaque
     * @param px the x coordinate of the point
     * @param py the y coordinate of the point
     * @return true if the point is on the plaque
     */
    private boolean onPlaque( double px, double py ) {
	return px >= offset_x && px <= offset_x+x_dim && 
	    py >= offset_y && py <= offset_y+y_dim;
    }

    private static final int pixel_gap = 1;

    /** Calculates the height of the pillar for a point on the plaque
     * @param px the x coordinate of the point
     * @param py the y coordinate of the point
     * @return the height of the pillar
     */
    private double pixelSurface( double tx, double ty) { 

	double step_x = (x_dim)/pixel_steps_x;
	double step_y = (y_dim)/(pixel_steps_y+2);
	//coordinates relative to plaque
	// how many pixels from left
	int pixLeft = (int) Math.floor((tx+0.5*step_x)/step_x);
	// how many pixels from bottom, ignoring pixel_gap boarder
	int pixUp = (int) Math.floor((ty+0.5*step_y)/step_y);
	if (pixUp < pixel_gap) return 0.0;
	pixUp -= pixel_gap;
	if(pixUp >= pixel_steps_y || pixLeft >= pixel_steps_x) return 0.0;
	if (pixels[pixUp][pixLeft]) 
	    return pixel_height;
	else return 0.0;
    }

}
