/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.surface.service;

import javax.xml.namespace.QName;

import org.globus.tutorial.types.surface.ProbeException;

import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.ArrayOfDouble;

import org.globus.wsrf.Resource;

import org.globus.tutorial.impl.randgen.service.Box;
import org.globus.tutorial.impl.randgen.service.BoxException;

import java.util.Properties;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;

import org.globus.wsrf.impl.SimpleResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertySet;

import org.globus.tutorial.surface.SurfaceResource;

public class Surface1Resource extends SurfaceResource {

    /** Generates the surface samples
     * @param input the input 2D data
     * @return the ouput 3D data
     */
    public Data genSurface(Data input) 
        throws BoxException, ProbeException{

        setupConfig();
	ArrayOfDouble[] arrays = input.getArray();
	if (arrays.length != 2) {
	    throw makeFault("need x and y, not " + arrays.length +" arrays");
        }

	double[] x = arrays[0].getValue();
	double[] y = arrays[1].getValue();

	if(x.length != y.length) {
	    throw makeFault(" not the same number of x and y ");
        }
	if(x.length > this.maxSampleSize) {
	    throw makeFault("maximum number of points: "+
                            this.maxSampleSize +" exceeded");
	}
	double[] z = new double[x.length];
	for(int i=0; i < x.length;i++) {
	    if(!this.bound.inBox(x[i],y[i])) {
                throw new BoxException("Point out of bounds "+x[i]+" "+y[i]);
            }
            z[i] = surfaceFunction(x[i],y[i]);
        }
        return new Data(new ArrayOfDouble[] {arrays[0], arrays[1],
					     new ArrayOfDouble(z)});
    }

    /** The surface function which returns the height of
     * a point on the surface for given 2D coordinates
     * @param x  the x coordinate
     * @param y  the y coordinate
     * @return the height of the surface
     */
    protected double surfaceFunction(double x, double y) {
        double[][] coeff = {{0,-1,1,-1},
			    {1,0,0,0},
			    {-1,0,0,0},
			    {1,0,0,0}};
        return polynomial(x,y,coeff);
    }

    /** Returns the result of the function 
     * sum_ij = a_ij x^i y^j  (i,j) integer >= 0
     * @param x the x coordinate for evaluating the function
     * @param y the y coordinate for evaluating the function
     * @return the result
     */
    private double polynomial(double x, double y, double[][] coeff){
        double sum = 0;
        double xval = 1;
        for(int i=0; i < coeff.length; i++) {
	    double yval = 1;
            for(int j=0; j < coeff[i].length; j++) {
                if(coeff[i][j] != 0) {
                    sum += coeff[i][j]*xval*yval;
                }
                yval*=y;
            }
            xval*=x;
        }
	return sum;
    }

    protected void setupConfig(Properties prop) throws ProbeException {
        
        double lowerX = new Double(prop.getProperty(LOWER_X)).doubleValue();
        double lowerY = new Double(prop.getProperty(LOWER_Y)).doubleValue();
        double upperX = new Double(prop.getProperty(UPPER_X)).doubleValue();
        double upperY = new Double(prop.getProperty(UPPER_Y)).doubleValue();
        try {
            this.bound = new Box(lowerX, lowerY, upperX, upperY);
        } catch (BoxException exp) {
            throw makeFault("Error setting up config", exp);
        }
    }
}
