/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.surface;

import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import java.util.Properties;

import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceProperties;
import org.globus.wsrf.ResourceProperty;
import org.globus.wsrf.ResourcePropertySet;

import javax.xml.namespace.QName;

import org.globus.tutorial.types.filestore.Data;

import org.globus.tutorial.impl.randgen.service.Box;
import org.globus.tutorial.impl.randgen.service.BoxException;

import org.globus.wsrf.impl.SimpleResourceProperty;
import org.globus.wsrf.impl.SimpleResourcePropertySet;

import org.globus.tutorial.types.surface.ProbeException;

import org.globus.wsrf.utils.FaultHelper;

public abstract class SurfaceResource implements Resource, ResourceProperties {

    public static final String MAX_SAMPLE = "maxSamples";
    public static final String LOWER_X = "lowerX";
    public static final String LOWER_Y = "lowerY";
    public static final String UPPER_X = "upperX";
    public static final String UPPER_Y = "upperY";

    protected int maxSampleSize;
    protected Box bound = null;
    protected File configFile;
    protected Long lastModified = null;

    public static final String NS = "http://tutorial.globus.org/surface";
    ResourcePropertySet propSet;
    QName RP_SET = new QName(NS, "SurfaceRPSet");

    /**
     * Initializes a surface resource reading in values for the maximum
     * sample size and the boundary from a config file
     * @param configFile_ the file containing the values for the
     * resource properties.
     */
    public void initialize(String configFile_) throws ProbeException {
        this.configFile = new File(configFile_);
        setupConfig();
    }

    // Resource properties
    public ResourcePropertySet getResourcePropertySet() {
        return this.propSet;
    }

    private void setResourceProperties() {

        this.propSet = new SimpleResourcePropertySet(RP_SET);
        QName qName = new QName(NS, "maxSampleSize");
        ResourceProperty prop = new SimpleResourceProperty(qName);
        prop.add(new Integer(this.maxSampleSize));
        this.propSet.add(prop);
        qName = new QName(NS, "boundBox");
        prop = new SimpleResourceProperty(qName);
        // FIXME: ugly. need common data types across all these
        // services after figuring out dependecy tree.
        prop.add(new org.globus.tutorial
                 .types.surface.Box(this.bound.getLowerCornerX(), 
                                    this.bound.getLowerCornerY(),
                                    this.bound.getUpperCornerX(),
                                    this.bound.getUpperCornerY()));
        this.propSet.add(prop);
    }

    public abstract Data genSurface(Data input) 
        throws BoxException, ProbeException;

    protected abstract void setupConfig(Properties prop) throws ProbeException;

    protected void setupConfig() throws ProbeException {

        Long ts = new Long(this.configFile.lastModified());
        if (this.lastModified != null) {
            if (ts.compareTo(this.lastModified) == 0) {
                return;
            }
        }
        Properties prop = new Properties();

        FileInputStream fis = null;
        try {
            fis = new FileInputStream(this.configFile);
            prop.load(fis);
            if (fis != null) {
                try {
                    fis.close();
                } catch (Exception e) {
		    System.out.println("problem closing");
		}
            }
        } catch (FileNotFoundException fnfe) {
            throw makeFault("Error reading config file", fnfe);
        } catch (IOException exp) {
            throw makeFault("Error reading config file", exp);
        } 

        this.maxSampleSize = 
            new Integer(prop.getProperty(MAX_SAMPLE)).intValue();
        
        setupConfig(prop);
        this.lastModified = ts;
        setResourceProperties();
    }

    public static ProbeException makeFault(String errMsg) {
        return makeFault(errMsg, null);
    }

    public static ProbeException makeFault(String errMsg, Exception exp) {
        ProbeException fault = new ProbeException();
        FaultHelper helper = new FaultHelper(fault);
        if (errMsg != null) {
            helper.setDescription(errMsg);
        }
        if (exp != null) {
            helper.addFaultCause(exp);
        }

        return (ProbeException)helper.getFault();
    }
}
