/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.visualizer.service;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import school.tutorial.gnu.XMLToGnuFormat;
import school.tutorial.gnu.XMLGnuConversionException;

import org.apache.axis.message.addressing.EndpointReferenceType;

import java.rmi.RemoteException;

import org.globus.tutorial.types.visualizer.RenderException;

import org.globus.tutorial.types.filestore.VoidType;
import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.ArrayOfDouble;
import org.globus.tutorial.filestore.FileService;
import org.globus.tutorial.filestore.service.FileServiceAddressingLocator;
import org.globus.tutorial.types.filestore.FileStoreException;

import javax.xml.rpc.ServiceException;
import java.util.Random;

import org.globus.wsrf.utils.FaultHelper;

/**
 * Class for visualising the sampled surface using GnuPlot
 */
public class Visualizer {

    public String render(EndpointReferenceType parameters) 
        throws RemoteException, RenderException, ServiceException {

	final String dir = "visualizer";
	String error = "";
	java.io.File root_dir = new java.io.File(dir);
	if(!root_dir.exists()) {
	    throw new RenderException("Visualiser not set up properly"+
				      " root directory for temp files"+
				      " does not exist");
        }
	try{
	    //fetch the data to be visualised
            FileServiceAddressingLocator loc = 
                new FileServiceAddressingLocator();
            FileService service = loc.getFileServicePort(parameters);
	    Data tdf = service.fetch(new VoidType());

	    //create temporary filenames for intermediate files
	    Random rand = new Random();
	    int i = rand.nextInt(10000);
	    String tempFile = dir+"/Temp"+i+".dat";
	    final String gnuScript = tempFile+"_gnu";

	    //convert the xml to standard text
	    XMLToGnuFormat xtg = new XMLToGnuFormat();
	    xtg.convert(tdf,tempFile);

	    //check no filename conflicts
	    java.io.File tempFGnu = new java.io.File(gnuScript);
	    if(tempFGnu.exists()) {
		throw new IOException("Temporary file for GnuPlot script "+
                                      "already exists. Try again");
            }

	    //write the gnuplot script to file
	    PrintWriter pw = new PrintWriter(new FileWriter(tempFGnu));
	    pw.println("set terminal post \n");
            pw.println("set boxwidth \n");
            pw.println("set missing '?'\n");
	    pw.println("set style fill empty border\n");
	    pw.println("set format x \"%.2f\n");
	    pw.println("set format y \"%.2f\n");
	    pw.println("unset grid\n");
	    pw.println("unset label\n");
            pw.println("unset arrow\n");
            pw.println("unset style line\n");
            pw.println("set pointsize 1\n");
            pw.println("set encoding default\n");
            pw.println("set origin 0,0\n");
            pw.println("set xzeroaxis lt -2 lw 1.000\n");
            pw.println("set yzeroaxis lt -2 lw 1.000\n");
            pw.println("set ticslevel 0.5\n");
            pw.println("set notitle\n");
            pw.println("set xlabel\n");
            pw.println("set ylabel\n");
            pw.println("set zlabel\n");
            pw.println("set zero 1e-008\n");
            pw.println("set colorbox default\n");
            pw.println("set loadpath\n");
            pw.println("set fontpath\n");
            pw.println("splot '" + tempFile + "' using 1:2:3 notitle\n");
            pw.println("#    EOF");
	    pw.close();

	    //execute gnuplot
	    String[] cmd = {"/usr/bin/gnuplot4", gnuScript};
	    Runtime rt = Runtime.getRuntime();
	    Process p = rt.exec(cmd);

	    //capture gnuplot output = postscript
	    BufferedReader br = new 
		BufferedReader(new InputStreamReader(p.getInputStream()));
	    StringBuffer sb = new StringBuffer();
	    String line;
	    while((line = br.readLine()) != null) {
		sb.append(line+"\n");
	    }
	    br.close();

	    //make sure process has finished
	    p.waitFor();

	    //delete temporary files
            java.io.File tempF = new java.io.File(tempFile);
            tempF.delete();
            tempFGnu.delete();
            
	    return sb.toString();
	} catch (FileStoreException fse) {
            FaultHelper fh = new FaultHelper(fse);
            String er = fh.getDescriptionAsString();
            if (er.equals("This operation can only be used with 3D files")) {
                error = "Visualizer: Input file should be a 3D file";
            } else {
                error = "Visualizer: "+er;
            }
        } catch(RemoteException re){
	    error = "Visualizer: Remote error accessing FileStore: "
                + re.getMessage();
	} catch(XMLGnuConversionException xge){
	    error = "Visualizer: "+xge.getMessage();
	} catch(InterruptedException ie){
	    error = "Visualizer: process executing gnuplot interrupted ";
	} catch(IOException ioe){
	    
	    error = "Visualizer: IOError "+ioe.getMessage();
	}
	throw new RenderException(error);
    }
}
