/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.impl.xprod.service;

import org.globus.tutorial.impl.seqgen.service.SeqGen;

import org.globus.tutorial.types.xprod.CrossResponse;
import org.globus.tutorial.types.xprod.CrossRequest;

import org.globus.tutorial.types.xprod.Request;
import org.globus.tutorial.types.xprod.Response;

import java.rmi.RemoteException;

import org.globus.tutorial.types.xprod.XprodGeneratorException;

import org.apache.axis.message.addressing.EndpointReferenceType;

import javax.xml.rpc.Stub;

import org.globus.tutorial.filestore.FileService;
import org.globus.tutorial.filestore.service.FileServiceAddressingLocator;
import org.globus.tutorial.types.filestore.FileStoreException;

import javax.xml.rpc.ServiceException;

import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.ArrayOfDouble;
import org.globus.tutorial.types.filestore.VoidType;

import org.globus.wsrf.utils.FaultHelper;

public class XprodService {


    /** Method for taking two sets of 1D data from input files
     * and outputting 2D data containing all possible pairs
     * of the 1D data sets.
     * @param parameters request object containing the input file
     * and output file EPRs
     * @return the response object.
     */         
    public Response generate(Request parameters) 
        throws XprodGeneratorException, ServiceException {

        EndpointReferenceType fileX = parameters.getFileX();
        EndpointReferenceType fileY = parameters.getFileY();
        EndpointReferenceType outputFile = parameters.getOutputFile();

	FileServiceAddressingLocator loc;
	FileService service;
	Data xData = null;
	try {
	    loc =  new FileServiceAddressingLocator();
	    service = loc.getFileServicePort(fileX);
	    xData = service.fetch(new VoidType());
	} catch (FileStoreException fse){
	    FaultHelper fh = new FaultHelper(fse);
	    String er = fh.getDescriptionAsString();
	    if (er.equals("This operation can only be used with 1D "
			  + "files")) {
		throw new XprodGeneratorException("Xprod: Input file should be"
                                                  + "a 1D file");
	    } else {
		throw new XprodGeneratorException("Xprod: FileStore: "+er);
	    }
	} catch(RemoteException re){
	    throw new XprodGeneratorException("Xprod: Remote exception: error "
                                              +" accessing  FileStore for "
                                              + " sequence one " 
                                              + re.getMessage());
	}

	Data yData = null;
	try {
	    service = loc.getFileServicePort(fileY);
	    yData = service.fetch(new VoidType());
	} catch (FileStoreException fse){
	    FaultHelper fh = new FaultHelper(fse);
	    String er = fh.getDescriptionAsString();
	    if (er.equals("This operation can only be used with 1D "
			  + "files")) {
		throw new XprodGeneratorException("Xprod: Input file should be"
                                                  + " a 1D file");
	    } else {
		throw new XprodGeneratorException("Xprod: FileStore: " + er);
	    }
	} catch(RemoteException re){
	    throw new XprodGeneratorException("Xprod: Remote exception: error"
                                              + " accessing FileStore for "
                                              + " sequence two " +
                                              re.getMessage());
	}

        Data retData = getCrossProduct(xData, yData);

        if (retData == null) {
            throw new XprodGeneratorException("Cross product data is null");
        }

	try {
	    service = loc.getFileServicePort(outputFile);
	    service.store(retData);
	    return new Response();
	} catch (FileStoreException fse){
	    FaultHelper fh = new FaultHelper(fse);
	    String er = fh.getDescriptionAsString();

	    if (er.equals("This operation can only be used with 2D files")) {
		throw new XprodGeneratorException("Xprod: Output file should "
                                                  + "be a 2D file");
	    } else {
		throw new XprodGeneratorException("Xprod: FileStore: "+er);
	    }
	} catch(RemoteException re){
	    throw new XprodGeneratorException("Xprod: Remote exception: error"
                                              + " accessing FileStore to store"
                                              + " Xprod Generation " 
                                              + re.getMessage());
	}
    }

    private Data getCrossProduct(Data xData, Data yData) 
        throws XprodGeneratorException {

        if ((xData == null) || (yData == null)) {
            throw new XprodGeneratorException("xData or yData is null");
        }

        if ((xData.getArray() == null) || (yData.getArray() == null) ||
            ((xData.getArray())[0] == null) || 
            ((yData.getArray())[0] == null)) {
            throw new XprodGeneratorException("xData or yData is null");
        }

        double[] x = (xData.getArray())[0].getValue();
        double[] y = (yData.getArray())[0].getValue();
        double[] xXprod = new double[x.length*y.length];
        double[] yXprod = new double[x.length*y.length];
        int k = 0;
        for(int i=0 ; i < x.length;i++) {
            for(int j=0; j< y.length;j++) {
                xXprod[k] = x[i];
                yXprod[k] = y[j];
                k++;
            }
        }
        return new Data(new ArrayOfDouble[]
            { new ArrayOfDouble(xXprod),
              new ArrayOfDouble(yXprod) });
    }
    
    public CrossResponse generateCrossProduct(CrossRequest parameters) 
        throws RemoteException, XprodGeneratorException {

        SeqGen gen = new SeqGen();
        //        System.out.println("Sequences: "+parameters.getStart1()+" "+parameters.getStep1()+" "+ parameters.getLimit1());
        Data seq1 = gen.getSequence(parameters.getStart1(),
                                    parameters.getStep1(),
                                    parameters.getLimit1());

        //        System.out.println("Sequences: "+parameters.getStart2()+" "+parameters.getStep2()+" "+ parameters.getLimit2());
        Data seq2 = gen.getSequence(parameters.getStart2(),
                                    parameters.getStep2(),
                                    parameters.getLimit2());

        Data output = getCrossProduct(seq1, seq2);

	try {
	    FileServiceAddressingLocator loc =
                new FileServiceAddressingLocator();
	    FileService service = 
                loc.getFileServicePort(parameters.getOutputFile());
	    service.store(output);
	} catch (Exception exp) {
            exp.printStackTrace();
            throw new XprodGeneratorException("Xprod: genCrossProduct" +
                                              exp.getMessage());
        }
        return new CrossResponse();
    }
}
